/**
 * Created by konstantin on 10/9/17.
 */

var promises = [];

$(function () {
    $('#submit-partner').click(submitPartner);
    $('#submit-import').click(submitImport);

    updateScrollspyHeight();

    $(window).on('resize', updateScrollspyHeight);

    $('[data-toggle="tooltip"]').tooltip();
});

/**
 * On submit import form handler
 */
function submitImport() {

    var $btn = $(this);

    var $progressBar = $('.progress-bar').css('width', '0%').attr('aria-valuenow', 0);
    $progressBar.parent().removeClass('hidden');
    var $progressBarContainer = $('.progress-col').removeClass('hidden');

    var triggers = [
        'trigger_new_file',
        'trigger_move_into',
        'trigger_download_hi',
        'trigger_download_fpo',
        'trigger_image_order',
        'trigger_image_preview',
        'trigger_date_expired'
    ];

    var $partnersJs = $('.partner-name');
    var totalRequests = $partnersJs.length + triggers.length;
    var i = 0;
    var stopLoading = false;

    /**
     * Add partners import to queue
     */
    $partnersJs.each(function() {

        /**
         * Import only checked
         */
        if(!$(this).parent().find('[id*="_import"]').prop('checked')) {
            return;
        }

        if (stopLoading) {
            $progressBar.parent().addClass('hidden');
            $btn.removeClass('disabled');
            $.ajaxq.abort('importPartner');
            return;
        }

        /**
         * Serialize form
         */
        var data = $(this).parent().find('input, textarea').serialize();

        var request = $.ajaxq('importPartner', {
            cache: false,
            data: data,
            type: 'POST',
            beforeSend: function () {
                var postData = this.data.split('&');
                if(postData.length) {
                    var partnerName = postData[0].split('=')[1];
                    $('#current-step').html('Importing partner: ' + partnerName);
                }
                $btn.addClass('disabled');
            },
            success: function (data) {

                if (!data) {
                    $.ajaxq.abort('importPartner');
                    $btn.removeClass('disabled');
                    $progressBar.parent().addClass('hidden');
                    return;
                }

                i = i + 1;
                var percentDone = parseInt((i / totalRequests) * 100);
                $progressBar.css('width', percentDone + '%').attr('aria-valuenow', percentDone).html(percentDone + '%');

                var result = $.parseJSON(data);

                // $('#result').append(result.result);
                console.log(result);

                /**
                 * Form validation errors
                 */
                var $partner = $('input[value="' + result.partner + '"]');
                var partnerContainer = $partner.parent();

                /**
                 * Validation errors container
                 */
                var validationErrorsContainer = $('#validation-errors');
                validationErrorsContainer.parent().addClass('hidden');

                if(result.formErrors) {
                    for (var field in result.formErrors) {

                        /**
                         * Open collapses
                         */
                        $partner.parent().find('[id*="' + field + '"]').removeAttr('readonly').addClass('is-invalid');
                        $partner.parent().find('.collapse').addClass('show');

                        /**
                         * Add validation error to the top warning alert
                         */

                        var errors = validationErrorsContainer.html();
                        validationErrorsContainer.html(errors + result.formErrors[field][0]);
                        validationErrorsContainer.parent().removeClass('hidden');

                        /**
                         * Add local validation error
                         */
                        var localValidationErrorsContainer = $partner.parent().find('.validation-errors');
                        var localErrors = localValidationErrorsContainer.html();
                        localValidationErrorsContainer.html(localErrors + result.formErrors[field][0]);
                        $partner.parent().find('.alert-warning').removeClass('hidden');

                    }
                }

                /**
                 * Import errors
                 */
                if(result.import) {

                    /**
                     * Show error/info messages
                     */
                    if(result.import.message) {
                        var importErrors = partnerContainer.find('.import-errors');
                        importErrors.html('');

                        partnerContainer.find('.card-text').addClass('hidden');

                        result.import.message.forEach(function (value) {
                            importErrors.append(value);
                        });
                    }

                    /**
                     * Uncheck import checkbox if import went without errors
                     */
                    if(result.import.error === false) {
                        partnerContainer.find('[name*="[import]"]').prop('checked', false);
                    }

                }

                stopLoading = result.import ? result.import.error : false;

                if (stopLoading) {
                    $progressBar.parent().addClass('hidden');
                    $.ajaxq.abort('importPartner');
                }

            }
        });

        promises.push(request);
    });

    /**
     * Add trigger creation to queue
     */
    $('#triggers').find('.card-text').html('');

    for (var j = 0; j < triggers.length; j++) {

        var curStep = triggers[j];

        if (stopLoading) {
            $progressBar.parent().addClass('hidden');
            $btn.removeClass('disabled');
            $.ajaxq.abort('importPartner');
            break;
        }

        var request = $.ajaxq('importPartner', {
            cache: false,
            data: {trigger: curStep},
            type: 'POST',
            beforeSend: function () {
                var postData = this.data.split('&');
                if(postData.length) {
                    var partnerName = postData[0].split('=')[1];
                    $('#current-step').html('Creating trigger: ' + partnerName.split('_').join(' '));
                }
                $btn.addClass('disabled');
            },
            success: function (data) {

                if (!data) {
                    $.ajaxq.abort('importPartner');
                    $btn.removeClass('disabled');
                    $progressBar.parent().addClass('hidden');
                    return;
                }

                i = i + 1;
                var percentDone = parseInt((i / totalRequests) * 100);
                $progressBar.css('width', percentDone + '%').attr('aria-valuenow', percentDone).html(percentDone + '%');

                var result = $.parseJSON(data);
                $('#triggers').removeClass('hidden').find('.card-text').append(result.result);

                stopLoading = result.error;

                if (stopLoading) {
                    $progressBar.parent().addClass('hidden');
                    $.ajaxq.abort('importPartner');
                }
            }
        });

        promises.push(request);
    }

    /**
     * When all AJAX is finished
     */
    $.when.apply($, promises).then(function () {
        $btn.removeClass('disabled');
        $('#current-step').html('');
        $progressBar.parent().addClass('hidden');
        $progressBarContainer.addClass('hidden');

        stopLoading = true;
        // $(btn).removeClass('btn-danger').html(loadValueLabel);

        /**
         * Redirect to partner edit configuration page if no alerts found
         */
        var $alerts = $('.alert-danger');
        if (!$alerts.length) {
            console.log('complete!');
            // window.location.href = $('#update-url').attr('href');
        }
    });


    return false;
}

function submitPartner() {

    var valid = true;
    var $form = $('#partner-form');

    // here I am checking for textFields, password fields, and any
    // drop down you may have in the form
    $("input[type='text'],select,input[type='password'], textarea").each(function () {
        if ($(this).val().trim() == "") {
            valid = false;
        }
    });

    /**
     * Post invalid form for Yii validation
     */
    if (!valid) {
        return true;
    }

    $('#result').html('');
    var $btn = $(this);


    var $progressBar = $('.progress-bar').css('width', '0%').attr('aria-valuenow', 0);
    $progressBar.parent().removeClass('hidden');
    var $progressBarContainer = $('.progress-col').removeClass('hidden');

    var steps = [
        'partner',
        'partner_folder',
        'to_partner',
        'from_partner',

        'trigger_new_file',
        'trigger_move_into',
        'trigger_download_hi',
        'trigger_download_fpo',
        'trigger_image_order',
        'trigger_image_preview',
        'trigger_date_expired'
    ];

    var totalRequests = steps.length;
    var i = 0;
    var stopLoading = false;

    for (var j = 0; j < steps.length; j++) {

        var curStep = steps[j];

        if (stopLoading) {
            $progressBar.parent().addClass('hidden');
            $btn.removeClass('disabled');
            $.ajaxq.abort('createPartner');
            break;
        }

        var data = $form.serializeArray(); // convert form to array
        data.push({name: 'step', value: curStep});

        var request = $.ajaxq('createPartner', {
            cache: false,
            data: $.param(data),
            type: 'POST',
            beforeSend: function () {
                // $btn.addClass('disabled');
                $('.invalid-feedback').hide();
            },
            success: function (data) {

                if (!data) {
                    $.ajaxq.abort('createPartner');
                    $btn.removeClass('disabled');
                    $progressBar.parent().addClass('hidden');
                    return;
                }

                var result = $.parseJSON(data);

                console.log(result);
                
                if(typeof result.error !== 'undefined' && result.error) {

                    for (var key in result.result) {

                        console.log(key);
                        console.log(result.result.hasOwnProperty(key));

                        if (result.result.hasOwnProperty(key)) {

                            var field = $('#PartnerForm_' + key);

                            if(field.length) {
                                field.next().html(result.result[key][0]).show();
                            }
                        }
                    }
                }

                $('#result').append(result.result);

                i = i + 1;
                var percentDone = parseInt((i / totalRequests) * 100);
                $progressBar.css('width', percentDone + '%').attr('aria-valuenow', percentDone).html(percentDone + '%');


                stopLoading = result.error;

                if (stopLoading) {
                    $progressBar.parent().addClass('hidden');
                    $.ajaxq.abort('createPartner');
                }

            }
        });

        promises.push(request);
    }

    /**
     * When all AJAX is finished
     */
    $.when.apply($, promises).then(function () {
        $btn.removeClass('disabled');
        $progressBar.parent().addClass('hidden');
        // $progressBarContainer.addClass('hidden');

        stopLoading = true;
        // $(btn).removeClass('btn-danger').html(loadValueLabel);

        /**
         * Redirect to partner edit configuration page if no alerts found
         */
        var $alerts = $('.alert-danger');
        if (!$alerts.length) {
            window.location.href = $('#update-url').attr('href');
        }
    });


    return false;
}

function updateScrollspyHeight() {
    var $partnerNavbar = $('#partner-navbar');
    var $partnerScrollspy = $('#scrollspy');
    var $partnerButtons = $('#partner-buttons');

    if (!$partnerNavbar.length || !$partnerScrollspy.length || !$partnerButtons.length) {
        return;
    }

    var navbarOffset = $partnerNavbar.offset().top +
        $partnerNavbar.height() +
        parseInt($partnerNavbar.css('padding-top')) +
        parseInt($partnerNavbar.css('padding-bottom'));

    var buttonsOffset = $partnerButtons.height() + parseInt($partnerButtons.css('margin-top'));

    var scrollspyHeight = $(window).height() -
        (navbarOffset + parseInt($partnerScrollspy.css('margin-top')) + buttonsOffset + 25);

    $partnerScrollspy.css({'height': scrollspyHeight + 'px'});
}

function afterValidate(form, data, hasError) {
    var $submitPartner = $('#submit-partner');
    console.log(hasError);
    if (hasError) {
        $submitPartner.addClass('disabled').off();
    } else {
        $submitPartner.removeClass('disabled').click(submitPartner);
    }
}

/**
 * jQuery yiiactiveform plugin file.
 *
 * @author Qiang Xue <qiang.xue@gmail.com>
 * @link http://www.yiiframework.com/
 * @copyright 2008-2010 Yii Software LLC
 * @license http://www.yiiframework.com/license/
 * @since 1.1.1
 */

(function ($) {
    /*
     * returns the value of the CActiveForm input field
     * performs additional checks to get proper values for checkbox / radiobutton / checkBoxList / radioButtonList
     * @param o object the jQuery object of the input element
     */
    var getAFValue = function (o) {
        var type,
            c = [];
        if (!o.length) {
            return undefined;
        }
        if (o[0].tagName.toLowerCase() === 'span') {
            o.find(':checked').each(function () {
                c.push(this.value);
            });
            return c.join(',');
        }
        type = o.attr('type');
        if (type === 'checkbox' || type === 'radio') {
            return o.filter(':checked').val();
        } else {
            return o.val();
        }
    };

    /**
     * yiiactiveform set function.
     * @param options map settings for the active form plugin. Please see {@link CActiveForm::options} for availablel options.
     */
    $.fn.yiiactiveform = function (options) {
        return this.each(function () {
            var settings = $.extend({}, $.fn.yiiactiveform.defaults, options || {}),
                $form = $(this);

            if (settings.validationUrl === undefined) {
                settings.validationUrl = $form.attr('action');
            }
            $.each(settings.attributes, function (i) {
                this.value = getAFValue($form.find('#' + this.inputID));
                settings.attributes[i] = $.extend({}, {
                    validationDelay: settings.validationDelay,
                    validateOnChange: settings.validateOnChange,
                    validateOnType: settings.validateOnType,
                    hideErrorMessage: settings.hideErrorMessage,
                    inputContainer: settings.inputContainer,
                    errorCssClass: settings.errorCssClass,
                    successCssClass: settings.successCssClass,
                    beforeValidateAttribute: settings.beforeValidateAttribute,
                    afterValidateAttribute: settings.afterValidateAttribute,
                    validatingCssClass: settings.validatingCssClass
                }, this);
            });
            $form.data('settings', settings);

            settings.submitting = false;  // whether it is waiting for ajax submission result
            var validate = function (attribute, forceValidate) {
                if (forceValidate) {
                    attribute.status = 2;
                }
                $.each(settings.attributes, function () {
                    if (this.value !== getAFValue($form.find('#' + this.inputID))) {
                        this.status = 2;
                        forceValidate = true;
                    }
                });
                if (!forceValidate) {
                    return;
                }

                if (settings.timer !== undefined) {
                    clearTimeout(settings.timer);
                }
                settings.timer = setTimeout(function () {
                    if (settings.submitting || $form.is(':hidden')) {
                        return;
                    }
                    if (attribute.beforeValidateAttribute === undefined || attribute.beforeValidateAttribute($form, attribute)) {
                        $.each(settings.attributes, function () {
                            if (this.status === 2) {
                                this.status = 3;
                                $.fn.yiiactiveform.getInputContainer(this, $form).addClass(this.validatingCssClass);
                            }
                        });
                        $.fn.yiiactiveform.validate($form, function (data) {
                            var hasError = false;
                            $.each(settings.attributes, function () {
                                if (this.status === 2 || this.status === 3) {
                                    hasError = $.fn.yiiactiveform.updateInput(this, data, $form) || hasError;
                                }
                            });
                            if (attribute.afterValidateAttribute !== undefined) {
                                attribute.afterValidateAttribute($form, attribute, data, hasError);
                            }
                        });
                    }
                }, attribute.validationDelay);
            };

            $.each(settings.attributes, function (i, attribute) {
                if (this.validateOnChange) {
                    $form.find('#' + this.inputID).change(function () {
                        validate(attribute, false);
                    }).blur(function () {
                        if (attribute.status !== 2 && attribute.status !== 3) {
                            validate(attribute, !attribute.status);
                        }
                    });
                }
                if (this.validateOnType) {
                    $form.find('#' + this.inputID).keyup(function () {
                        if (attribute.value !== getAFValue($(this))) {
                            validate(attribute, false);
                        }
                    });
                }
            });

            if (settings.validateOnSubmit) {
                $form.on('mouseup keyup', ':submit', function () {
                    $form.data('submitObject', $(this));
                });
                var validated = false;
                $form.submit(function () {
                    if (validated) {
                        validated = false;
                        return true;
                    }
                    if (settings.timer !== undefined) {
                        clearTimeout(settings.timer);
                    }
                    settings.submitting = true;
                    if (settings.beforeValidate === undefined || settings.beforeValidate($form)) {
                        $.fn.yiiactiveform.validate($form, function (data) {
                            var hasError = false;
                            $.each(settings.attributes, function () {
                                hasError = $.fn.yiiactiveform.updateInput(this, data, $form) || hasError;
                            });
                            $.fn.yiiactiveform.updateSummary($form, data);
                            if (settings.afterValidate === undefined || settings.afterValidate($form, data, hasError)) {
                                if (!hasError) {
                                    validated = true;
                                    var $button = $form.data('submitObject') || $form.find(':submit:first');
                                    // TODO: if the submission is caused by "change" event, it will not work
                                    if ($button.length) {
                                        $button.click();
                                    } else {  // no submit button in the form
                                        $form.submit();
                                    }
                                    return;
                                }
                            }
                            settings.submitting = false;
                        });
                    } else {
                        settings.submitting = false;
                    }
                    return false;
                });
            }

            /*
             * In case of reseting the form we need to reset error messages
             * NOTE1: $form.reset - does not exist
             * NOTE2: $form.on('reset', ...) does not work
             */
            $form.bind('reset', function () {
                /*
                 * because we bind directly to a form reset event, not to a reset button (that could or could not exist),
                 * when this function is executed form elements values have not been reset yet,
                 * because of that we use the setTimeout
                 */
                setTimeout(function () {
                    $.each(settings.attributes, function () {
                        this.status = 0;
                        var $error = $form.find('#' + this.errorID),
                            $container = $.fn.yiiactiveform.getInputContainer(this, $form);

                        $container.removeClass(
                            this.validatingCssClass + ' ' +
                            this.errorCssClass + ' ' +
                            this.successCssClass
                        );

                        $error.html('').hide();

                        /*
                         * without the setTimeout() we would get here the current entered value before the reset instead of the reseted value
                         */
                        this.value = getAFValue($form.find('#' + this.inputID));
                    });
                    /*
                     * If the form is submited (non ajax) with errors, labels and input gets the class 'error'
                     */
                    $form.find('label, :input').each(function () {
                        $(this).removeClass(settings.errorCss);
                    });
                    $('#' + settings.summaryID).hide().find('ul').html('');
                    //.. set to initial focus on reset
                    if (settings.focus !== undefined && !window.location.hash) {
                        $form.find(settings.focus).focus();
                    }
                }, 1);
            });

            /*
             * set to initial focus
             */
            if (settings.focus !== undefined && !window.location.hash) {
                $form.find(settings.focus).focus();
            }
        });
    };

    /**
     * Returns the container element of the specified attribute.
     * @param attribute object the configuration for a particular attribute.
     * @param form the form jQuery object
     * @return jQuery the jQuery representation of the container
     */
    $.fn.yiiactiveform.getInputContainer = function (attribute, form) {
        if (attribute.inputContainer === undefined) {
            return form.find('#' + attribute.inputID).closest('div');
        } else {
            return form.find(attribute.inputContainer).filter(':has("#' + attribute.inputID + '")');
        }
    };

    /**
     * updates the error message and the input container for a particular attribute.
     * @param attribute object the configuration for a particular attribute.
     * @param messages array the json data obtained from the ajax validation request
     * @param form the form jQuery object
     * @return boolean whether there is a validation error for the specified attribute
     */
    $.fn.yiiactiveform.updateInput = function (attribute, messages, form) {
        attribute.status = 1;
        var $error, $container,
            hasError = false,
            $el = form.find('#' + attribute.inputID),
            errorCss = form.data('settings').errorCss;

        if ($el.length) {
            hasError = messages !== null && $.isArray(messages[attribute.id]) && messages[attribute.id].length > 0;
            $error = form.find('#' + attribute.errorID);
            $container = $.fn.yiiactiveform.getInputContainer(attribute, form);

            $container.removeClass(
                attribute.validatingCssClass + ' ' +
                attribute.errorCssClass + ' ' +
                attribute.successCssClass
            );
            $container.find('label, :input').each(function () {
                $(this).removeClass(errorCss);
            });

            if (hasError) {
                $error.html(messages[attribute.id][0]);
                $container.addClass(attribute.errorCssClass);
            } else if (attribute.enableAjaxValidation || attribute.clientValidation) {
                $container.addClass(attribute.successCssClass);
            }
            if (!attribute.hideErrorMessage) {
                $error.toggle(hasError);
            }

            attribute.value = getAFValue($el);
        }
        return hasError;
    };

    /**
     * updates the error summary, if any.
     * @param form jquery the jquery representation of the form
     * @param messages array the json data obtained from the ajax validation request
     */
    $.fn.yiiactiveform.updateSummary = function (form, messages) {
        var settings = $(form).data('settings'),
            content = '';
        if (settings.summaryID === undefined) {
            return;
        }
        if (messages) {
            var summaryAttributes = [];
            for (var i in settings.attributes) {
                if (settings.attributes[i].summary) {
                    summaryAttributes.push(settings.attributes[i].id);
                }
            }
            $.each(settings.attributes, function () {
                if ($.inArray(this.id, summaryAttributes) !== -1 && $.isArray(messages[this.id])) {
                    $.each(messages[this.id], function (j, message) {
                        content = content + '<li>' + message + '</li>';
                    });
                }
            });
        }
        $('#' + settings.summaryID).toggle(content !== '').find('ul').html(content);
    };

    /**
     * Performs the ajax validation request.
     * This method is invoked internally to trigger the ajax validation.
     * @param form jquery the jquery representation of the form
     * @param successCallback function the function to be invoked if the ajax request succeeds
     * @param errorCallback function the function to be invoked if the ajax request fails
     */
    $.fn.yiiactiveform.validate = function (form, successCallback, errorCallback) {
        var $form = $(form),
            settings = $form.data('settings'),
            needAjaxValidation = false,
            messages = {};
        $.each(settings.attributes, function () {
            var value,
                msg = [];
            if (this.clientValidation !== undefined && (settings.submitting || this.status === 2 || this.status === 3)) {
                value = getAFValue($form.find('#' + this.inputID));
                this.clientValidation(value, msg, this);
                if (msg.length) {
                    messages[this.id] = msg;
                }
            }
            if (this.enableAjaxValidation && !msg.length && (settings.submitting || this.status === 2 || this.status === 3)) {
                needAjaxValidation = true;
            }
        });

        if (!needAjaxValidation || settings.submitting && !$.isEmptyObject(messages)) {
            if (settings.submitting) {
                // delay callback so that the form can be submitted without problem
                setTimeout(function () {
                    successCallback(messages);
                }, 200);
            } else {
                successCallback(messages);
            }
            return;
        }

        var $button = $form.data('submitObject'),
            extData = '&' + settings.ajaxVar + '=' + $form.attr('id');
        if ($button && $button.length) {
            extData += '&' + $button.attr('name') + '=' + $button.attr('value');
        }

        $.ajax({
            url: settings.validationUrl,
            type: $form.attr('method'),
            data: $form.serialize() + extData,
            dataType: 'json',
            success: function (data) {
                if (data !== null && typeof data === 'object') {
                    $.each(settings.attributes, function () {
                        if (!this.enableAjaxValidation) {
                            delete data[this.id];
                        }
                    });
                    successCallback($.extend({}, messages, data));
                } else {
                    successCallback(messages);
                }
            },
            error: function () {
                if (errorCallback !== undefined) {
                    errorCallback();
                }
            }
        });
    };

    /**
     * Returns the configuration for the specified form.
     * The configuration contains all needed information to perform ajax-based validation.
     * @param form jquery the jquery representation of the form
     * @return object the configuration for the specified form.
     */
    $.fn.yiiactiveform.getSettings = function (form) {
        return $(form).data('settings');
    };

    $.fn.yiiactiveform.defaults = {
        ajaxVar: 'ajax',
        validationUrl: undefined,
        validationDelay: 200,
        validateOnSubmit: false,
        validateOnChange: true,
        validateOnType: false,
        hideErrorMessage: false,
        inputContainer: undefined,
        errorCss: 'error',
        errorCssClass: 'error',
        successCssClass: 'success',
        validatingCssClass: 'validating',
        summaryID: undefined,
        timer: undefined,
        beforeValidateAttribute: undefined, // function (form, attribute) | boolean
        afterValidateAttribute: undefined,  // function (form, attribute, data, hasError)
        beforeValidate: undefined, // function (form) | boolean
        afterValidate: undefined,  // function (form, data, hasError) | boolean
        focus: undefined,  // jquery selector that indicates which element to receive input focus initially
        /**
         * list of attributes to be validated. Each array element is of the following structure:
         * {
		 *     id: 'ModelClass_attribute', // the unique attribute ID
		 *     model: 'ModelClass', // the model class name
		 *     name: 'name', // attribute name
		 *     inputID: 'input-tag-id',
		 *     errorID: 'error-tag-id',
		 *     value: undefined,
		 *     status: 0,  // 0: empty, not entered before,  1: validated, 2: pending validation, 3: validating
		 *     validationDelay: 200,
		 *     validateOnChange: true,
		 *     validateOnType: false,
		 *     hideErrorMessage: false,
		 *     inputContainer: undefined,
		 *     errorCssClass: 'error',
		 *     successCssClass: 'success',
		 *     validatingCssClass: 'validating',
		 *     enableAjaxValidation: true,
		 *     enableClientValidation: true,
		 *     clientValidation: undefined, // function (value, messages, attribute) | client-side validation
		 *     beforeValidateAttribute: undefined, // function (form, attribute) | boolean
		 *     afterValidateAttribute: undefined,  // function (form, attribute, data, hasError)
		 * }
         */
        attributes: []
    };
})(jQuery);
