<?php
/**
 * @category   NAPC Elegant 6.1
 * @package    elegant6.1
 * @author     Kirill Krasnobayev <kirill.krasnobayev@napc.com>
 * @copyright  © 2019 NAPC
 * @version    6.1
 */
class ActionDownload extends CAction
{

    /**
     * @param nTransitPlugin $widget
     * @throws Exception
     */
    public function run($widget)
    {
       $id = $this->validateInitialParams();

        /**
         * if user is guest, store current url and redirect to login page
         * (ths ueser will be redirected back here after login)
         */
        if (Yii::app()->user->isGuest) {
            UrlHelper::setReturnUrl();
            Yii::app()->getController()->redirect(Yii::app()->getController()->createUrl('/login'));
        }

        /**
         * check if asset exists
         */
        $this->validateAsset($id);

        /**
         * redirect to "Download Basket" plugin, passing newly create basket path
         */
        $this->redirectToPlugin($this->prepareBasket($id));

    }

    /**
     * @return int
     */
    private function validateInitialParams() {

        /**
         * get asset id value
         */
        $id = Yii::app()->request->getParam('id', null);

        /**
         * get hash value
         */
        $hash = Yii::app()->request->getParam('hash', null);

        /**
         * if hash and is are not available - show error
         */
        if (!$id || !$hash) {
            $this->error('Invalid data passed');
        }

        /**
         * decrypt hash
         */
        $hash = Helper::decryptRef('hash');

        /**
         * if hash is not equal to id - show error
         */
        if (!is_array($hash) || count($hash) != 1 ||
            $hash[0] !== $id) {
            $this->error('Invalid hash passed');
        }

        return $id;

    }

    /**
     * @param int $id
     */
    private function validateAsset($id) {

        /**
         * get file data from portalDI
         */
        $fileData = PortalDI::getInstance()->callPortal(PortalDI::ACTION_FILEINFO, array('fileid' => $id));

        /**
         * if data is not available - show error
         */
        if(!$fileData || !isset($fileData['FILES_INFO']) || !is_array($fileData['FILES_INFO']) ||
            count($fileData['FILES_INFO']) === 0) {
            $this->error('Asset does not exist or is inaccessible');
        }

    }

    private function prepareBasket($id)
    {

        /**
         * prepare name for new basket
         */
        $basketName = 'ntransit_5.' . microtime(true);

        /**
         * put asset to basket
         */
        $basketData = PortalDI::getInstance()->callPortal(PortalDI::ACTION_ADD_TO_BASKET, 'basketname=' . $basketName, '&fileid=' . $id);

        /**
         * if basket data does not contain that asset - show error
         */
        if(!$basketData || !isset($basketData['BASKET_INFO']) || !is_array($basketData['BASKET_INFO']) ||
            count($basketData['BASKET_INFO']) === 0) {
            $this->error('Could not add asset to basket');
        }

        /**
         * return basket name
         */
        return $basketName;

    }

    /**
     * @param string $basketName
     */
    private function redirectToPlugin($basketName) {

        /**
         * redirect to "Download Basket" plugin
         */
        Yii::app()->getController()->redirect(Yii::app()->getController()->createUrl('/plugin/downloadbasket') .
            '?plugin-url=' . $this->getBasketPath() . $basketName . '.basket&standalone=true');

    }

    private function getBasketPath()
    {

        /**
         * get message sender basket data from portalDI
         */
        $diUser = PortalDI::getInstance()->callPortal(PortalDI::ACTION_SHOWUSER_SETTINGS);

        $currentUserBasket = @$diUser['BASKETFILE'];

        /**
         * if we got the basket, use it's path, otherwise create generic path
         */
        if ($currentUserBasket) {
            $currentUserBasket = pathinfo($currentUserBasket, PATHINFO_DIRNAME) . '/';
        } else {
            $currentUserBasket = '/var/adm/webnative/' . PortalDI::getInstance()->getUsername() . '/';
        }

        /**
         * return basket path
         */
        return $currentUserBasket;

    }

    /**
     * @param string $message
     */
    private function error($message)
    {

        $this->controller->layout = '//layouts/simple';
        $this->controller->render('application.plugins.nTransitPlugin.views.download_error', array('error' => $message));

        exit;

    }


}