<?php
/**
 * Created by PhpStorm.
 * User: konstantin
 * Date: 1/30/18
 * Time: 1:30 PM
 */

class ActionHealthcheck extends CAction
{

    public function run($widget)
    {
        $widget->controller->pageTitle = $widget->pluginName . ' • Health check';

        $grandResult = null;
        
        /**
         * Database integrity check SQLite
         */
        $dbFile = dirname(__FILE__)  . '/../../../data/ntransit.db';

        if(!file_exists($dbFile)) {
            $grandResult .= NtransitHelper::getStatus('Database file not found "' . $dbFile . '"', true);
        } else {
            $grandResult .= NtransitHelper::getStatus('Database file found');

            $db = new CDbConnection('sqlite:' . $dbFile);

            $integrityCheck = null;
            try {
                $integrityCheck = $db->createCommand('PRAGMA integrity_check')->queryAll();
            } catch (CDbException $exception) {
                $grandResult .= NtransitHelper::getStatus($exception->getMessage(), true);
            }

            if(is_array($integrityCheck) && isset($integrityCheck[0]['integrity_check']) &&
                $integrityCheck[0]['integrity_check'] === 'ok') {
                $grandResult .= NtransitHelper::getStatus('Database integrity check PASSED');

            } elseif(is_array($integrityCheck) && isset($integrityCheck[0]['integrity_check']) &&
                $integrityCheck[0]['integrity_check'] !== 'ok') {
                $grandResult .= NtransitHelper::getStatus('Database integrity check results: ' .print_r($integrityCheck), true);
            }
        }

        /**
         * Check if a user has access to T/A management
         */
        $portalDI = PortalDI::getInstance();
        $portalDI->setUsername($widget->getSetting('username'));
        $portalDI->setPassword($widget->getSetting('password'));

        $triggerName = 'nTransit - New File';

        $result = $portalDI->callPortal(
            PortalDI::ACTION_USERPERMS,
            array('-ts+' . rawurlencode($triggerName) . '+-trigsetedit'),
            array('raw' => true, 'postData' => null, 'timeout' => 30)
        );

        preg_match('/Permission denied/mi', $result, $matches);

        if(is_array($matches) && count($matches)) {
            $grandResult .= NtransitHelper::getStatus('No Trigger/Actions access, check user permissions', true);
        } else {
            $grandResult .= NtransitHelper::getStatus('Trigger/Actions access granted');
        }

        /**
         * Check if a user has access to ntransit volume
         */
        UrlHelper::prepareMainUrls();

        $userVolumes = $portalDI->callPortal(PortalDI::ACTION_SHOW_VOLUMES);
        $volumeAccess = false;
        foreach ($userVolumes as $volume) {
            if($volume === $widget->volume) {
                $volumeAccess = true;
                break;
            }
        }

        if($volumeAccess) {
            $grandResult .= NtransitHelper::getStatus('No nTransit volume access, check user permissions', true);
        } else {
            $grandResult .= NtransitHelper::getStatus('nTransit volume access granted');
        }



        /**
         * Check partner volumes (root, from and to partner)
         */
        $partners = Partner::model()->findAll();
        $foldersAllSet = true;
        foreach ($partners as $partner) {

            if(!$this->checkAccess(array(
                $widget->volume . DIRECTORY_SEPARATOR . $partner->name,
                $widget->volume . DIRECTORY_SEPARATOR . $partner->name . DIRECTORY_SEPARATOR . $partner->from_partner,
                $widget->volume . DIRECTORY_SEPARATOR . $partner->name . DIRECTORY_SEPARATOR . $partner->to_partner,
            ), $grandResult, $portalDI)) {
                $foldersAllSet = false;
            }
        }

        if($foldersAllSet) {
            $grandResult .= NtransitHelper::getStatus('Partner volumes and paths are set');
        }

        return $widget->render('standalone', array(
            'content' => $widget->render('healthcheck', array(
                'data' => $grandResult,
            )),
            'model' => null,
            'breadcrumbs' => $widget->render('breadcrumbs', array('LIST' => NtransitHelper::getBreadcrumbs(array(
                CHtml::link('Home', 'nTransitPlugin'),
                'Health check'
            )))),
            'header' => '<i class="fa fa-medkit" aria-hidden="true"></i> Health check',
            'imgPath' => $widget->imgFolder,
        ));
    }

    /**
     * Check if Webnative folder exists and user has access
     * @param array $pathArray
     * @param string $result
     * @param PortalDI $portalDI
     * @return bool
     */
    private function checkAccess($pathArray, &$result, $portalDI)
    {

        $success = true;
        foreach ($pathArray as $path) {

            $params = array(
                'action' => 'fileinfo',
                'path' => $path,
                'showkywds' => 'false'
            );

            /**
             * execute portalDI call
             */
            $checkResult = $portalDI->callPortal(PortalDI::ACTION_FILEINFO, $params);

            /**
             * process error messages
             */
            if(!(isset($checkResult['FILES_INFO']) && is_array($checkResult['FILES_INFO']) && count($checkResult['FILES_INFO']))) {
                $result .= NtransitHelper::getStatus('Folder "' . $path
                    . '" doesn\'t exists or user has no access', true);

                $success = false;
            }
        }

        return $success;

    }


}