<?php
/**
 * Created by PhpStorm.
 * User: konstantin
 * Date: 1/29/18
 * Time: 3:28 PM
 */

class ActionSendnotification extends CAction
{

    const RESULT_WRONG_PAYLOAD = 'PAYLOAD_CANNOT_BE_READ';
    const RESULT_ASSET_IGNORED = 'ASSET_IGNORED_INSIDE_FOLDER';
    const RESULT_CANNOT_GET_PARTNER = 'CANNOT_GET_PARTNER_FROM_SERVER';
    const RESULT_NO_PATH = 'NO_FILE_PATH_PROVIDED';
    const RESULT_CANNOT_SEND = 'CANNOT_SEND_EMAILS';
    const RESULT_SUCCESS = 'SUCCESS';
    const RESULT_ASSET_NOT_FOUND = 'ASSET_NOT_FOUND_ON_SERVER';
    const RESULT_EXPIRED = 'EXPIRED';

    private $username;
    private $password;
    private $folderLifespanKeyword;
    private $folderLifespanKeywordId;
    private $events;
    private $widget;

    private $payloadTriggerSet;
    private $payloadUser;
    private $payloadFileId;
    private $path;

    /**
     * @param $widget
     * @throws Exception
     */
    public function run($widget)
    {
        $this->events = NtransitHelper::getEvents();
        $this->username = $widget->getSetting('username');
        $this->password = $widget->getSetting('password');
        $this->folderLifespanKeyword = $widget->getSetting('folderLifespanKeyword');
        $this->folderLifespanKeywordId = $widget->getSetting('folderLifespanKeywordId');
        $this->widget = $widget;

        if(!($this->path = Yii::app()->request->getParam('path', null))) {
            $this->done(self::RESULT_NO_PATH);
        }
        
        $payload = Helper::decryptRef('payload');

        if (count($payload) !== 4) {
            $this->done(self::RESULT_WRONG_PAYLOAD);
        } else {
            list($this->payloadTriggerSet, $this->payloadUser, $this->payloadFileId, $timestamp) = $payload;

            if(time() > $timestamp) {
                $this->done(self::RESULT_EXPIRED);
            }
        }

        $parentName = null;

        /**
         * Example: /raid/nTransit/Ogilvy/to_partner/photo_2017-12-07_15-21-46.jpg
         * 1. remove nTransit volume:
         *      1.1 /Ogilvy/to_partner/photo_2017-12-07_15-21-46.jpg or
         *      1.2 Ogilvy/to_partner/photo_2017-12-07_15-21-46.jpg
         * 2. Explode and find first not empty array item
         */
        $pathArray = explode('/', str_replace($widget->volume, '', $this->path));

        /**
         * 3. Remove empty array values in case 1.1
         */
        $pathArray = array_values(array_filter($pathArray));

        /**
         * If filtered array has one or less items then something went wrong
         */
        if (count($pathArray) < 1) {
            $this->done('ERROR_PARSING ' . print_r($pathArray, true));
        }

        $parentName = reset($pathArray);

        /**
         * 4. Try to find partner by its name
         */
        $partnerSearch = Partner::model()->findByAttributes(array('name' => $parentName));

        if (!$partnerSearch) {

            $this->done('CANNOT_FIND_PARTNER ' . $parentName);

        }

        if (empty($partnerSearch->config)) {

            $this->done('UNCONFIGURED_PARTNER ' . $partnerSearch->name);
        }

        $configDecoded = json_decode($partnerSearch->config);

        /**
         * Send notification via Swiftmailer
         */
        $sendResult = $this->sendEmail(
            $partnerSearch,
            $configDecoded,
            $pathArray,
            $this->payloadTriggerSet);

        /**
         * Set expire data field
         */
        if ($this->payloadTriggerSet == NtransitHelper::EVENT_NEW_FILE || $this->payloadTriggerSet == NtransitHelper::EVENT_MOVE_INTO) {
            $this->setExpireDate($this->path, $configDecoded);
        }

        $this->done($sendResult, $sendResult !== true);
    }

    /**
     * @param Partner $partner
     * @param stdClass $config
     * @param array $pathArray
     * @param string $trigger
     * @return bool
     * @throws Exception
     */
    private function sendEmail($partner, $config, $pathArray, $trigger)
    {

        file_put_contents(dirname(__FILE__) . '/sendresult', '1' . PHP_EOL, FILE_APPEND);
        /**
         * If user drops a folder in from/to partner and Partner configured to send only folders
         * then we bypass send action for files that are not on top level of partner to/from folders
         */
        if ($this->bypassSendOnlyFolders($partner, $pathArray)) {
            return self::RESULT_ASSET_IGNORED;
        }

        $this->logEvent($partner, $trigger);

        switch ($trigger) {

            /**
             * New file email notification
             */
            case NtransitHelper::EVENT_NEW_FILE:
            case NtransitHelper::EVENT_MOVE_INTO:

                if ($pathArray[1] == $partner->to_partner) {

                    $assetLinks = $this->getAssetLinks($config, $partner, $pathArray);

                    $message = <<<MESSAGE
{$config->sentMessage}
<br/><br/>
{$assetLinks['open']} | {$assetLinks['download']}

MESSAGE;

                    $mailer = new nTransitMailer(
                        $config->sentFrom,
                        $config->sentName,
                        $config->sentRecipients,
                        $config->sentCCRecipients,
                        $config->sentBCCRecipients,
                        $config->sentSubject,
                        $message,
                        $this->widget);

                } elseif ($pathArray[1] == $partner->from_partner) {

                    $assetLinks = $this->getAssetLinks($config, $partner, $pathArray);

                    $message = <<<MESSAGE
{$config->receivedMessage}
<br/><br/>
{$assetLinks['open']} | {$assetLinks['download']}

MESSAGE;

                    $mailer = new nTransitMailer(
                        $config->receivedFrom,
                        $config->receivedName,
                        $config->receivedRecipients,
                        $config->receivedCCRecipients,
                        $config->receivedBCCRecipients,
                        $config->receivedSubject,
                        $message,
                        $this->widget);
                } else {
                    return self::RESULT_CANNOT_GET_PARTNER;
                }


                return $mailer->send() > 0 || self::RESULT_CANNOT_SEND;

            /**
             * Download notification
             */
            case NtransitHelper::EVENT_DOWNLOAD_HIRES:
            case NtransitHelper::EVENT_DOWNLOAD_FPO:
            case NtransitHelper::EVENT_IMAGE_ORDER:
            case NtransitHelper::EVENT_IMAGE_PREVIEW:

                $assetLinks = $this->getAssetLinks($config, $partner, $pathArray);

                if ($trigger === NtransitHelper::EVENT_DOWNLOAD_HIRES ||
                    $trigger === NtransitHelper::EVENT_DOWNLOAD_FPO) {
                    $messageContent = "Someone downloaded file(s):";
                } else if ($trigger === NtransitHelper::EVENT_IMAGE_ORDER) {
                    $messageContent = "Someone performed custom image download of file(s):";
                } else {
                    $messageContent = "Someone previewed file(s):";
                }

                $message = <<<MESSAGE
$messageContent
<br/><br/>
{$assetLinks['open']} | {$assetLinks['download']}

MESSAGE;

                $mailer = new nTransitMailer(
                    'donotreply@ntransit.com',
                    null,
                    $config->downloadRecipients,
                    null,
                    null,
                    'Download notification',
                    $message,
                    $this->widget);

                return $mailer->send() > 0 || self::RESULT_CANNOT_SEND;

        }


    }

    /**
     * @param Partner $partner
     * @param array $pathArray
     * @return bool
     */
    private function bypassSendOnlyFolders($partner, $pathArray)
    {

        /**
         * remove partner name from path array
         */
        if (($key = array_search($partner->name, $pathArray)) !== false) {
            unset($pathArray[$key]);
        }

        /**
         * remove partner to/from folders from path array
         */
        if (($key = array_search($partner->to_partner, $pathArray)) !== false) {
            unset($pathArray[$key]);
        }

        if (($key = array_search($partner->from_partner, $pathArray)) !== false) {
            unset($pathArray[$key]);
        }

        /**
         * If after stripping partner name and partner from/to folders we have more than
         * one item in path array than it's not top level folder and we bypass it
         */
        $result = count($pathArray) > 1;

        return $result;
    }

    /**
     * @param stdClass $config
     * @param Partner $partner
     * @param array $pathArray
     * @return null|array
     */
    private function getAssetLinks($config, $partner, $pathArray)
    {
        $result = null;

        $assetPath = $partner->volume . DIRECTORY_SEPARATOR . implode(DIRECTORY_SEPARATOR, $pathArray);

        /**
         * Determine is it a file or a folder
         */
        $portalDI = PortalDI::getInstance();
        $portalDI->setUsername($this->username);
        $portalDI->setPassword($this->password);
        list($file) = PortalDI::getInstance()->callPortal(
            PortalDI::ACTION_FILEINFO, array('fileid' => $this->payloadFileId, 'advanced' => true));

        if (!$file) {
            $this->done(self::RESULT_ASSET_NOT_FOUND);
        }

        $action = $file->isDir() ? 'browse' : 'view';

        $siteLink = Yii::app()->createAbsoluteUrl($config->partnerHasSite ? $partner->name : '') . ($config->partnerHasSite ? '/' : '');

        $openLink = "$siteLink$action?id=$file->id";

        $hash = Encryption::getInstance()->encrypt($file->id);

        $downloadLink = "{$siteLink}plugin/nTransitPlugin?action=download&id=$file->id&hash=$hash";

        return array(
            'open' => CHtml::link($assetPath, $openLink),
            'download' => CHtml::link('download', $downloadLink),
        );

    }

    /**
     * @param string $path
     * @param array $config
     */
    private function setExpireDate($path, $config)
    {

        Yii::log('Setting expire date...');

        $portalDI = PortalDI::getInstance();
        $portalDI->setUsername($this->username);
        $portalDI->setPassword($this->password);

        $keywordList = $portalDI->callPortal(PortalDI::ACTION_KEYWORDS);
        if (isset($keywordList[$this->folderLifespanKeywordId]) && $keyword = $keywordList[$this->folderLifespanKeywordId]) {

            /**
             * Bind file ID with keyword object
             */
            $keyword->setAssetId((int)$this->payloadFileId);

            /**
             * Set keyword value according to partner config
             */
            $expireDate = date('Y-m-d+H:i:s', strtotime('+' . $config->folderLifespan . ' minute'));
            $keyword->setValue($expireDate);

            /**
             * Save keyword, bypass validation, because it's already formatted for WN date post
             */
            $result = $keyword->save(false);

            if (isset($result['FILES_INFO']) && is_array($result['FILES_INFO']) && count($result['FILES_INFO'])) {
                Yii::log('Expire date is set to ' . $expireDate . ' ' . $path);
            }

        } else {
            Yii::log('Keyword ' . $this->folderLifespanKeyword . ' is not accessible for the current user');
        }
    }


    /**
     * @param Partner $partner
     * @param int $trigger
     */
    private function logEvent($partner, $trigger = 0)
    {

        $triggerLabel = isset($this->events[$trigger]) ? $this->events[$trigger] : null;

        $newLog = new Log();
        $newLog->type = $trigger ? (int)$trigger : 0; // event ID
        $newLog->date = time();

        $newLog->path = $this->path;
        $newLog->fileid = (int)$this->payloadFileId;
        $newLog->partner = $partner->name;
        $newLog->message = $triggerLabel . ($this->payloadFileId ? ', file ID: ' . $this->payloadFileId : '');
        $newLog->user = $this->payloadUser;
        if (!$newLog->save()) {
            Yii::log('Can\'t save log ' . print_r($newLog->getErrors(), true), CLogger::LEVEL_ERROR, get_class($this));
        }
    }

    private function done($message = null, $error = true)
    {

        $output = array('error' => $error);

        if ($message === true) {
            $output['data'] = self::RESULT_SUCCESS;
        } else if ($message !== null) {
            $output['data'] = $message;
        }

        exit(json_encode($output));
    }


}