<?php
/**
 * Created by PhpStorm.
 * User: konstantin
 * Date: 1/9/18
 * Time: 6:55 PM
 */

class NtransitHelper
{

    const EVENT_NEW_FILE = '8';
    const EVENT_MOVE_INTO = '2097152';
    const EVENT_DOWNLOAD_HIRES = '262144';
    const EVENT_DOWNLOAD_FPO = '524288';
    const EVENT_IMAGE_ORDER = '2048';
    const EVENT_IMAGE_PREVIEW = '1048576';
    const EVENT_DATE_EXPIRED = '-1';

    const EVENT_NEW_FILE_LABEL = 'New File';
    const EVENT_MOVE_INTO_LABEL = 'Move Into';
    const EVENT_DOWNLOAD_HIRES_LABEL = 'Download High Res';
    const EVENT_DOWNLOAD_FPO_LABEL = 'Download FPO';
    const EVENT_IMAGE_ORDER_LABEL = 'Image Order';
    const EVENT_IMAGE_PREVIEW_LABEL = 'Preview Viewed';
    const EVENT_DATE_EXPIRED_LABEL = 'Date Expired';

    public static function getEvents()
    {
        return array(
            self::EVENT_NEW_FILE => self::EVENT_NEW_FILE_LABEL,
            self::EVENT_MOVE_INTO => self::EVENT_MOVE_INTO_LABEL,
            self::EVENT_DOWNLOAD_HIRES => self::EVENT_DOWNLOAD_HIRES_LABEL,
            self::EVENT_DOWNLOAD_FPO => self::EVENT_DOWNLOAD_FPO_LABEL,
            self::EVENT_IMAGE_ORDER => self::EVENT_IMAGE_ORDER_LABEL,
            self::EVENT_IMAGE_PREVIEW => self::EVENT_IMAGE_PREVIEW_LABEL,
            self::EVENT_DATE_EXPIRED => self::EVENT_DATE_EXPIRED_LABEL,
        );
    }

    // Sending exception to Google Analytics
    public static function ga($action)
    {
        $message = 'nTransit 5 ' . $action;

        Yii::app()->ServerSideGoogleAnalytics->sendException($message . ' ' . $_SERVER['REQUEST_URI'] . ' ' .
            (isset($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : ''), 0);
    }

    /**
     * Render breadcrumbs list
     * @param array $breadcrumbs
     * @return null|string
     */
    public static function getBreadcrumbs($breadcrumbs = array())
    {
        $list = null;
        foreach ($breadcrumbs as $breadcrumb) {
            $list .= CHtml::tag('li', array('class' => 'breadcrumb-item'), $breadcrumb);
        }
        return $list;
    }

    /**
     * @param string $message
     * @param bool $success
     * @return string mixed
     */
    public static function getStatus($message, $error = false)
    {

        if ($error === true) {
            return CHtml::tag('div', array('class' => 'alert alert-danger'),
                '<i class="fa fa-exclamation" aria-hidden="true"></i> ' . $message);
        } else {
            return CHtml::tag('div', array('class' => 'alert alert-success'),
                '<i class="fa fa-check" aria-hidden="true"></i> ' . $message);
        }
    }

    /**
     * Check if Webnative partner folder exists and create it if not
     * @param string $path
     * @param string $folderName
     * @return string
     */
    public static function createPartnerFolder($path, $folderName, $widget, $returnResults = false)
    {
        $partnerFolder = $path . DIRECTORY_SEPARATOR . $folderName;

        sleep(1);

        $result = '';
        $error = false;
        if (!self::folderExists($path, $folderName, $widget)) {


            if (($resultPartnerFolder = self::createFolder($path, $folderName, $widget)) !== false) {
                $error = true;
                $result .= self::getStatus(' Can\'t create partner folder <span class="badge badge-primary ml-2">' .
                    $partnerFolder . '</span> ' . $resultPartnerFolder, $error);
            } else {
                $result .= self::getStatus(' <span class="badge badge-primary ml-2">' . $partnerFolder .
                    '</span> partner folder created');
            }

        } else {
            $result .= self::getStatus(' <span class="badge badge-primary ml-2">' . $partnerFolder .
                '</span> partner folder exists, continue');
        }

        if (Yii::app()->request->isAjaxRequest && !$returnResults) {
            echo json_encode(array('result' => $result, 'error' => $error));

            Yii::app()->end();
        } else {
            return array('result' => $result, 'error' => $error);
        }
    }

    /**
     * @param array $result
     * @return bool
     */
    public static function getErrorMessage($result)
    {
        $errorMessage = false;
        if (!$result)
            $errorMessage = true;

        if (isset($result['ERRCODE']) && ((int)$result['ERRCODE'] < 200 || (int)$result['ERRCODE'] > 299))
            $errorMessage = isset($result['MSG']) ? $result['MSG'] : null;

        if (isset($result['ERROR_MSG']))
            $errorMessage = $result['ERROR_MSG'];

        return $errorMessage;
    }

    /**
     * Create blank event trigger
     * @param string $eventName
     */
    public static function createTrigger($eventName, $widget)
    {

        $triggerSet = "nTransit - $eventName";

        /**
         * Check if trigger already exists
         */
        if (self::triggerExists($triggerSet, $widget)) {
            return false;
        }

        $triggerSetDesc = "nTransit 5 trigger for $eventName Event";

        $postData = array(
            'fromtrigsetadd' => '',
            'trigset_newname' => $triggerSet,
            'trigset_newdesc' => $triggerSetDesc,
            'trigset_copy' => '-None-',
            'trigsetcreateandedit' => 'Add and Edit',
        );

        $portalDI = PortalDI::getInstance();
        $portalDI->setUsername($widget->getSetting('username'));
        $portalDI->setPassword($widget->getSetting('password'));

        $portalDI->callPortal(
            PortalDI::ACTION_USERPERMS,
            null,
            array('raw' => true, 'postData' => http_build_query($postData), 'timeout' => 30)
        );

        return true;
    }

    /**
     * Add event to existing trigger
     * @param string $eventType
     * @param string $eventName
     */
    public static function addTriggerEvent($eventType, $eventName, $widget)
    {

        $triggerSet = "nTransit - $eventName";
        $triggerSetDesc = "nTransit 5 trigger for $eventName Event";

        $portalDI = PortalDI::getInstance();
        $portalDI->setUsername($widget->getSetting('username'));
        $portalDI->setPassword($widget->getSetting('password'));

        /**
         * Add trigger file event
         */
        $postData = array(
            'fromtrigsetedit' => 'UserPerms',
            'trigset_name' => $triggerSet,
            'trigset_desc' => $triggerSetDesc,
            'filetrigger' => $eventType,
            'trigsetaddfilerule' => 'Add',
        );

        /**
         * Add data field, time based event trigger
         */
        if ($eventType == '-1') {
            /**
             * Remove file trigger fields from POST array
             */
            unset($postData['filetrigger']);
            unset($postData['trigsetaddfilerule']);

            $postData['keyword_name'] = $widget->getSetting('folderLifespanKeyword');
            $postData['trigsetaddrule'] = 'Add';
        }

        $result = $portalDI->callPortal(
            PortalDI::ACTION_USERPERMS,
            null,
            array('raw' => true, 'postData' => http_build_query($postData), 'timeout' => 30)
        );

        /**
         * Get added event ID
         */
        preg_match_all('/ruleactionitem\(\d+/', $result, $matches, PREG_SET_ORDER, 0);

        $eventId = null;
        if (count($matches) == 1 && isset($matches[0][0])) {
            $eventId = str_replace('ruleactionitem(', '', $matches[0][0]);
        } elseif (count($matches) == 2 && isset($matches[1][0])) {
            $eventId = str_replace('ruleactionitem(', '', $matches[1][0]);
        }

        /**
         * If can't get event ID throw an error and exit;
         */
        if (!$eventId) {
            $result = array('error' => true, 'result' => NtransitHelper::getStatus('Can\'t get trigger event ID', true));
            if (Yii::app()->request->isAjaxRequest) {
                echo json_encode($result);

                Yii::app()->end();
            } else {
                return $result;
            }
        }

        /**
         * Get added trigger ID
         */
        preg_match_all('/trigruleitem\(\d+/', $result, $matches, PREG_SET_ORDER, 0);

        $triggerId = null;
        if (count($matches) == 1 && isset($matches[0][0])) {
            $triggerId = str_replace('trigruleitem(', '', $matches[0][0]);
        } elseif (count($matches) == 2 && isset($matches[1][0])) {
            $triggerId = str_replace('trigruleitem(', '', $matches[1][0]);
        }

        /**
         * If can't get trigger ID throw an error and exit;
         */
        if (!$triggerId) {
            $result = array('error' => true, 'result' =>
                NtransitHelper::getStatus('Can\'t get trigger ID', true));

            if (Yii::app()->request->isAjaxRequest) {
                echo json_encode($result);

                Yii::app()->end();
            } else {
                return $result;
            }

        }

        /**
         * Link trigger file event and trigger action
         */
        $postData = array(
            'fromtrigsetedit' => 'UserPerms',
            'trigset_name' => $triggerSet,
            'trigset_desc' => $triggerSetDesc,
            'action_aname' . $eventId => $widget->getSetting('actionName'),
            'action_sname' . $eventId => $widget->getSetting('actionSettings'),
            'action_nfail' . $eventId => 1,
            'trigsetsave' => 'Save',
        );

        /**
         * Add time based date data field rule
         */
        if ($eventType == '-1') {
            $postData['trigrule_delay' . $triggerId] = 0;
        }

        $portalDI->callPortal(
            PortalDI::ACTION_USERPERMS,
            null,
            array('raw' => true, 'postData' => http_build_query($postData), 'timeout' => 30)
        );
    }

    /**
     * Add trigger hot folder
     * @param string $volume
     * @param string $eventName
     */
    public static function addTriggerFolder($volume, $eventName, $widget)
    {
        $triggerSet = "nTransit - $eventName";
        $triggerSetDesc = "nTransit 5 trigger for $eventName Event";

        $portalDI = PortalDI::getInstance();
        $portalDI->setUsername($widget->getSetting('username'));
        $portalDI->setPassword($widget->getSetting('password'));

        /**
         * Prepare new path, and get its ID
         */
        $result = $portalDI->callPortal(
            PortalDI::ACTION_USERPERMS,
            array('-ts+' . rawurlencode($triggerSet) . '+-trigsetedit'),
            array('raw' => true, 'postData' => null, 'timeout' => 30)
        );

        preg_match_all('/{id:\d+/', $result, $matches, PREG_SET_ORDER, 0);

        $error = false;
        $pathId = null;
        if (isset($matches[0][0])) {
            $pathId = str_replace('{id:', '', $matches[0][0]);

            /**
             * Set path by its ID
             */
            $postData = array(
                'fromtrigsetedit' => 'UserPerms',
                'trigset_name' => $triggerSet,
                'trigset_desc' => $triggerSetDesc,
                'activepath_changed' . $pathId => 'path' . $pathId,
                'activepath_volpath' . $pathId => $volume,
                'activepath_subpath' . $pathId => '',
                '_subpath' . $pathId => '',
                'activepath_enabled' . $pathId => '1',
                'enabledcheckbox' . $pathId => 'on',
                'trigsetsave' => 'Save'
            );

            $portalDI->callPortal(
                PortalDI::ACTION_USERPERMS,
                null,
                array('raw' => true, 'postData' => http_build_query($postData), 'timeout' => 30)
            );

//            $result = NtransitHelper::getStatus('Trigger updated: ' . $triggerSet . ' volume linked');

        } else {
            $result = array('error' => true,
                'result' => NtransitHelper::getStatus('Can\'t add new path for: ' . $triggerSet, true));

            if (Yii::app()->request->isAjaxRequest) {
                echo json_encode($result);
                Yii::app()->end();
            } else {
                return $result;
            }

        }
    }

    /**
     * Get dedicated username from nTransit config
     * @return string|null
     */
    public static function getUsername() {
        $result = null;

        $config = Config::model()->find(array('order'=>'id ASC'));
        if($config) {
            $decoded = json_decode($config->data);
            $result = $decoded->username;
        }

        return $result;
    }

    /**
     * Check if Webnative folder exists
     * @param string $path
     * @param string $folderName
     * @return bool
     */
    private static function folderExists($path, $folderName, $widget)
    {
        $params = array(
            'action' => 'fileinfo',
            'path' => $path . DIRECTORY_SEPARATOR . $folderName,
            'showkywds' => 'false'
        );

        /**
         * execute portalDI call
         */
        UrlHelper::prepareMainUrls();
        $portalDI = PortalDI::getInstance();
        $portalDI->setUsername($widget->getSetting('username'));
        $portalDI->setPassword($widget->getSetting('password'));

        $result = $portalDI->callPortal(PortalDI::ACTION_FILEINFO, $params);

        if(isset($result['FILES_INFO'])) {
            error_log(print_r($result['FILES_INFO'], true));
        }

        /**
         * process error messages
         */
        return isset($result['FILES_INFO']) && is_array($result['FILES_INFO']) && count($result['FILES_INFO']) > 0;
    }

    /**
     * Create Webnative folder via PortalDI
     * @param string $path
     * @param string $folderName
     * @return bool
     */
    private static function createFolder($path, $folderName, $widget)
    {
        $params = array(
            'action' => 'filemgr',
            'filemgraction' => 'mkdir',
            'path' => $path,
            'newpath' => $path,
            'newname' => rawurlencode($folderName),
            'overwrite' => false,
        );

        /**
         * execute portalDI call
         */
        $portalDI = PortalDI::getInstance();
        $portalDI->setUsername($widget->getSetting('username'));
        $portalDI->setPassword($widget->getSetting('password'));

        $result = $portalDI->callPortal(PortalDI::ACTION_FILE_MGR, $params);

        error_log(print_r($result, true));

        sleep(3);

        /**
         * process error messages
         */
        return self::getErrorMessage($result);
    }

    /**
     * Checks if trigger exists by its name
     * @param string $triggerName
     * @return bool
     */
    private static function triggerExists($triggerName, $widget)
    {
        /**
         * Prepare new path, and get its ID
         */
        $portalDI = PortalDI::getInstance();
        $portalDI->setUsername($widget->getSetting('username'));
        $portalDI->setPassword($widget->getSetting('password'));

        $result = $portalDI->callPortal(
            PortalDI::ACTION_USERPERMS,
            array('-ts+' . rawurlencode($triggerName) . '+-trigsetedit'),
            array('raw' => true, 'postData' => null, 'timeout' => 30)
        );

        preg_match_all('/' . $triggerName . '/', $result, $matches, PREG_SET_ORDER, 0);

        return count($matches) > 0;
    }

    /**
     * Get config setting
     * @param $setting string
     * @return null|string|array
     */
    public static function getSetting($setting = null) {

        $config = Config::model()->find(array('order'=>'id ASC'));
        if($config) {
            $config = json_decode($config->data, true);
        }

        if($setting === null) {
            return isset($config) ? $config : array();
        } else {
            return isset($config) && isset($config[$setting]) ? $config[$setting] : null;
        }
    }
}