<?php
/**
 * @category   NAPC Elegant 6.1
 * @package    elegant6.1
 * @author     Konstantin Komarov <kirill.krasnobayev@napc.com>
 * @copyright  © 2017 NAPC
 * @version    6.1
 * @link       https://napcdev.git.beanstalkapp.com/elegant6_1.git
 */

/**
 * Class nTransitMailer
 */
class nTransitMailer
{

    /**
     * @var string $sender
     */
    protected $sender;

    /**
     * @var string $sender
     */
    protected $senderName;

    protected $recipients;
    protected $recipientsCC;
    protected $recipientsBCC;

    /**
     * @var null|string
     */
    protected $subject = null;
    /**
     * @var null|string
     */
    protected $content = null;

    /**
     * @var array $details
     */
    protected $details;

    /**
     * @var bool $async
     */
    protected $async = false;

    protected $widget;


    /**
     * nTransitMailer constructor.
     * @param string $from
     * @param string $to
     * @param string $ccTo
     * @param string $bccTo
     * @param string $subject
     * @param string $message
     */
    public function __construct($from, $fromName, $to, $ccTo, $bccTo, $subject, $message, $widget)
    {
        $this->sender = $from ? $from : "donotreply@ntransit.com";
        $this->senderName = $fromName;
        $this->recipients = str_replace(" ", "", $to);
        $this->recipientsCC = str_replace(" ", "", $ccTo);
        $this->recipientsBCC = str_replace(" ", "", $bccTo);
        $this->subject = $subject;
        $this->content = $message;
        $this->widget = $widget;

        $this->connectMailer();
    }

    /**
     * @return Swift_Mailer
     */
    private function connectMailer()
    {
        spl_autoload_unregister(array('YiiBase', 'autoload'));
        Yii::import('application.vendor.swift.swift_required', true);
        spl_autoload_register(array('YiiBase', 'autoload'));
    }

    /**
     * @inheritdoc
     */
    public function send()
    {

        /**
         * if recipient email is invalid - quit
         */
        if (!$this->emailValid($this->recipients) ||
            (isset($this->recipientsCC) && !empty($this->recipientsCC) && !$this->emailValid($this->recipientsCC)) ||
            (isset($this->recipientsBCC) && !empty($this->recipientsBCC) && !$this->emailValid($this->recipientsBCC))
        ) {
            throw new Exception('Invalid recipients');
        }

//        $this->formatSubject(
//            str_replace('{BANK}', $this->user->bank->getSite()->title,
//                JsonSettingsGetter::bankSettings("settings.subject", $this->user->bank)->value)
//        );

        return $this->execute();

    }

    /**
     * @param string $recipient
     * @return int
     */
    protected function execute()
    {

        /**
         * @var Swift_Mailer $mailer
         */
        $mailer = Swift_Mailer::newInstance($this->getTransport());

        /**
         * @var Swift_Message $message
         */
        $message = Swift_Message::newInstance();

        $this->setMessageBody($message);

        $message->setFrom($this->sender, $this->senderName);

        $message->setTo(explode(',', $this->recipients));

        /**
         * Set CC recipients
         */
        if (isset($this->recipientsCC) && !empty($this->recipientsCC)) {
            $message->setCc(explode(',', $this->recipientsCC));
        }

        /**
         * Set BCC recipients
         */
        if (isset($this->recipientsBCC) && !empty($this->recipientsBCC)) {
            $message->setBcc(explode(',', $this->recipientsBCC));
        }

        return $this->useMailer($mailer, $message);
    }

    /**
     * @param Swift_Mailer $mailer
     * @param Swift_Message $message
     * @return integer|null
     */
    private function useMailer($mailer, $message)
    {

        try {

            if ($this->async) {
                $this->sendThroughSocket($mailer, $message);
                return 1;

            } else {
                return $mailer->send($message, $failedRecipients);
            }

        } catch (Swift_TransportException $ex) {

            Yii::log($ex->getMessage(), 'error', 'sendMail');
            return null;
        }

    }

    /**
     * send email by passing data to send_mail.php through socket and immediately closing connection
     * @param Swift_Mailer $mailer
     * @param Swift_Message $message
     * @return void
     */
    private function sendThroughSocket($mailer, $message)
    {

        $path = Yii::app()->getBaseUrl(true) . '/send_mail';

        $parts = parse_url($path);

        $postData = http_build_query(array('data' => base64_encode(serialize(array(
            'mailer' => $mailer,
            'message' => $message
        )))));


        $handle = fsockopen($parts['host'], isset($parts['port']) ? $parts['port'] : 80, $errno, $errstr, 1);

        fputs($handle, "POST $path HTTP/1.0\r\n");
        fputs($handle, "Content-type: application/x-www-form-urlencoded\r\n");
        fputs($handle, "Content-length: " . strlen($postData) . "\r\n");

        fputs($handle, "Connection:close\r\n\r\n");

        fputs($handle, $postData);

        /**
         * close handler
         */
        fclose($handle);

    }


    /**
     * @return Swift_Transport_AbstractSmtpTransport
     */
    private function getTransport()
    {

        $transportType = (int)JsonSettingsGetter::globalSettings('mail_transport')->value;

        switch ($transportType) {
            case 1:
                $server = JsonSettingsGetter::globalSettings('smtp_server')->value;
                if (!$server) {
                    $server = 'localhost';
                }

                $port = (int)JsonSettingsGetter::globalSettings('smtp_port')->value;
                if (!$port) {
                    $port = 25;
                }

                $username = JsonSettingsGetter::globalSettings('smtp_user')->value;
                $password = JsonSettingsGetter::globalSettings('smtp_password')->value;
                return Swift_SmtpTransport::newInstance($server, $port)
                    ->setUsername($username)
                    ->setPassword($password);
            default:

                $command = JsonSettingsGetter::globalSettings('sendmail_command')->value;

                if (!$command) {
                    $command = '/usr/sbin/sendmail -bs';
                }

                return Swift_SendmailTransport::newInstance($command);

        }
    }

    /**
     * @inheritdoc
     */
    protected function attachContentDetails()
    {


//        switch ($this->type) {
//
//            case SendMailFactory::TYPE_ADMIN_REGISTRATION_MESSAGE:
//
//                if(is_string($this->user->optional_fields)) {
//                    $this->user->optional_fields = json_decode($this->user->optional_fields, true);
//                }
//                $this->details[Yii::t('backend', 'email.details.email')] = $this->user->email;
//                $this->details = array_merge($this->details, CreativeBanksHelper::prepareOptionalFieldsForEmail($this->user->bank, $this->user->optional_fields));
//
//                break;
//            case SendMailFactory::TYPE_ADMIN_USER_LOCKED_MESSAGE:
//
//
//                $this->details[Yii::t('backend', 'email.details.failed_attempts')] = $this->user->failed_attempts_count;
//
//                $unlockAfter = JsonSettingsGetter::bankSettings('security_settings.access_control_unlock', $this->user->bank)->value;
//                $this->details[Yii::t('backend', 'email.details.re-enabled_period')] = Yii::t('backend',
//                    $unlockAfter = isset(BankSettings::$userLockPeriods[$unlockAfter]) ? BankSettings::$userLockPeriods[$unlockAfter] :
//                        BankSettings::$userLockPeriods[1]);
//                break;
//
//        }
//
//        $this->details[Yii::t('backend', 'email.details.link')] = $this->user->bank->getSite()->prepareSiteLink('login', true);
//        $this->details[Yii::t('backend', 'email.details.edit_link')] = Yii::app()->createAbsoluteUrl("admin/cb-user/update/{$this->user->id}");
//
        return 'test'; //$this->wrapDetails();

    }

    /**
     * @param string $template
     * @param Swift_Message $message
     */
    private function setMessageBody($message)
    {

//        $content = $this->attachContentDetails();
        $message->setSubject($this->subject);

        $imgPath = Yii::app()->assetManager->publish(dirname(__FILE__) . '/../../../img', true);

        $body = $this->widget->render('email', array(
            'subject' => 'nTransit 5',
            'content' => $this->content,
            'imgPath' => $imgPath,
        ), true);

        $body = FileHelper::reformatImageUrlsForEmail($body, $message);

        $message->setBody(
            $body,
            'text/html'
        );

    }

    /**
     * @inheritdoc
     */
    protected function isTypeValid($type)
    {

        return true; //in_array($type, SendMailFactory::$validAdminTypes);
    }

    /**
     * @param $email
     * @return bool
     * @throws Exception
     */
    protected function emailValid($email)
    {

        $validator = new CEmailValidator();
        $emailArray = explode(',', $email);

        if (is_array($emailArray) && count($emailArray)) {

            foreach ($emailArray as $item) {
                if (!$validator->validateValue(trim($item))) {
                    throw new Exception('Invalid email: ' . $email);
                }
            }
        }

        return true;
    }

}